//
// Bitplane
// A bitplane can be seen as a paletted image, but without a palette.
//

#include "renderlib.h"
#include "surface.h"
#include "bitplane.h"
#include "bitstream.h"


// Load an Amiga format sprite from a bitstream
//
Bitplane* EXPORT bitplaneLoadAmigaSprite(Bitstream* Bits, const uint32_t Width, const uint32_t Height)
{

	uint32_t iX, X, Y, Plane;
	Bitplane* BP;


	if (Width % 16)
		return NULL;

	BP = bitplaneCreate(Width, Height, 4);

	// Each multiple of 16
	for (iX = 0; iX < Width; iX += 16)
	{
		// Every 2 bitplanes
		for (Plane = 0; Plane < 4; Plane += 2)
		{
			for (Y = 0; Y < Height; Y++)
			{
				for (X = 0; X < 32; X++)
				{
					if (bitsGetBits(Bits, 1))
					{
						if (X < 16)
							BP->ptrData[Y * Width + X + iX] |= (1 << Plane);
						else
							BP->ptrData[Y * Width + X + iX - 16] |= (1 << (Plane + 1));
					}
				}
			}
		}
	}

	return BP;

}


// Load a planar (Y x Plane x Y) bitplane from a bitstream
//
Bitplane* EXPORT bitplaneLoadPlanar(Bitstream* Bits, const uint32_t Width, const uint32_t Height, const uint8_t Planes)
{

	uint32_t X, Y, Plane;
	Bitplane* BP;


	if (Planes > 7)
		return NULL;

	BP = bitplaneCreate(Width, Height, Planes);

	for (Y = 0; Y < Height; Y++)
	{
		for (Plane = 0; Plane < Planes; Plane++)
		{
			for (X = 0; X < Width; X++)
			{
				if (bitsGetBits(Bits, 1))
					BP->ptrData[Y * Width + X] |= (1 << Plane);
			}
		}
	}

	return BP;

}


// Load a chunky (Plane x Y x X) bitplane from a bitstream
//
Bitplane* EXPORT bitplaneLoadChunky(Bitstream* Bits, const uint32_t Width, const uint32_t Height, const uint8_t Planes)
{

	uint32_t X, Y, Plane;
	Bitplane* BP;


	if (Planes > 7)
		return NULL;

	BP = bitplaneCreate(Width, Height, Planes);

	for (Plane = 0; Plane < Planes; Plane++)
	{
		for (Y = 0; Y < Height; Y++)
		{
			for (X = 0; X < Width; X++)
			{
				if (bitsGetBits(Bits, 1))
					BP->ptrData[Y * Width + X] |= (1 << Plane);
			}
		}
	}

	return BP;

}


// Create a new empty bitplane
//
Bitplane* EXPORT bitplaneCreate(const uint32_t Width, const uint32_t Height, const uint8_t Planes)
{

	Bitplane* BP;


	if (Planes > 7)
		return NULL;

	BP = malloc(sizeof(Bitplane));
	BP->Width = Width;
	BP->Height = Height;
	BP->Planes = Planes;
	BP->dataLen = Width * Height * sizeof(uint8_t);
	BP->ptrData = calloc(1, BP->dataLen);

	return BP;

}


// Destroy a bitplane
//
void EXPORT bitplaneDestroy(Bitplane* BP)
{

	if (!BP)
		return;

	free(BP->ptrData);
	free(BP);

	BP = NULL;

}


// Convert a bitplane to a surface
//
Surface* EXPORT bitplaneToSurface(const Bitplane* BP, Bitplane* Mask, const uint32_t* Pal, uint32_t transColor, const uint32_t Mode)
{

	uint32_t Pix;
	Surface* Surf;

 
	if (!BP)
		return NULL;

	Surf = surfaceCreate(BP->Width, BP->Height);

	// Convert the bitplane data to BGRA
	for (Pix = 0; Pix < BP->dataLen; Pix++)
		Surf->ptrData[Pix] = Pal[(uint8_t)BP->ptrData[Pix]] | (0xFF << 24);

	// Transparency by mask image if present
	if (Mode == MASKMODE_MASK && Mask)
	{
		if (BP->Width != Mask->Width || BP->Height != Mask->Height)
			return NULL;

		for (Pix = 0; Pix < BP->dataLen; Pix++)
			if (!Mask->ptrData[Pix])
				Surf->ptrData[Pix] &= 0x00FFFFFF;
	}

	// Color key transparency
	else if (Mode == MASKMODE_INDEX)
	{
		for (Pix = 0; Pix < BP->dataLen; Pix++)
			if (BP->ptrData[Pix] == transColor)
				Surf->ptrData[Pix] &= 0x00FFFFFF;
	}

	return Surf;

}