//
// Bitstream reader
// Reads data into memory and can access it by datatype up to the bit level.
//

#include "renderlib.h"
#include "bitstream.h"
#include "utils.h"



// Return size of bitstream data
//
uint32_t EXPORT bitsSize(Bitstream* Bits)
{

	if (!Bits)
		return 0;

	return Bits->dataLen;

}


// Return if EOF has been reached
//
uint32_t EXPORT bitsEOF(Bitstream* Bits)
{

	if (!Bits)
		return 0;

	return Bits->Byte == Bits->dataLen;

}


// Return the current byte offset inside a stream
//
uint32_t EXPORT bitsGetAddress(Bitstream* Bits)
{

	if (!Bits)
		return 0;

	return Bits->Byte;

}


// Read an integer from the bistream
//
uint32_t EXPORT bitsGetInt(Bitstream* Bits)
{

    uint32_t Val;


	if (!Bits)
		return 0;

	if (Bits->Byte > Bits->dataLen - 4)
		return 0;

	memcpy(&Val, Bits->ptrData + Bits->Byte, 4);
    bitsSkipBytes(Bits, 4);

	return SWAP4(Val);

}


// Read a short from the bistream
//
uint16_t EXPORT bitsGetShort(Bitstream* Bits)
{

    uint16_t Val;


	if (Bits->Byte > Bits->dataLen - 2)
		return 0;

	if (!Bits)
		return 0;

    memcpy(&Val, Bits->ptrData + Bits->Byte, 2);
    bitsSkipBytes(Bits, 2);

    return SWAP2(Val);

}


// Read a byte from the bistream
//
uint8_t EXPORT bitsGetByte(Bitstream* Bits)
{

    uint8_t Val;


	if (!Bits)
		return 0;

    memcpy(&Val, Bits->ptrData + Bits->Byte, 1);
    bitsSkipBytes(Bits, 1);

    return Val;

}


// Read a number of bits from the bitstream
// Maximum number of bits is 32
//
uint32_t EXPORT bitsGetBits(Bitstream* Bits, const uint8_t Count)
{

    uint32_t Bit;
    uint32_t Val = 0;


	if (!Bits)
		return 0;

    for(Bit = 0; Bit < Count; Bit++)
    {
		if (*(uint8_t*)(Bits->ptrData + Bits->Byte) & (1 << (7 - Bits->Bit)))
			Val |= (1 << (Count - 1 - Bit));

		// Advance bits
		Bits->Bit++;
		if (Bits->Bit > 7)
		{
			Bits->Bit = 0;
			if (Bits->Byte == Bits->dataLen)
				return Val;

			bitsSkipBytes(Bits, 1);
		}
    }

    return Val;

}


// Create a new empty bitstream
//
Bitstream* EXPORT bitsCreate()
{

    Bitstream* Bits;


    Bits = malloc(sizeof(Bitstream));
	Bits->dataLen = 0;
	Bits->Byte = 0;
	Bits->Bit = 0;
	Bits->ptrData = NULL;

    return Bits;

}


// Destroy a bitstream
//
void EXPORT bitsDestroy(Bitstream* Bits)
{

	if (!Bits)
		return;

	free(Bits->ptrData);
	free(Bits);

	Bits = NULL;

}


// Open a new bitstream from a buffer of data
//
Bitstream* EXPORT bitsOpenBuffer(const void* ptrData, const uint32_t dataLen)
{

    Bitstream* Bits;


    Bits = bitsCreate();
	if (!Bits)
		return NULL;

    // Copy data into our own buffer
	Bits->dataLen = dataLen;
    Bits->ptrData = malloc(dataLen);
    memcpy(Bits->ptrData, ptrData, dataLen);

    return Bits;

}


// Seek to a byte position in the bitstream
//
uint32_t EXPORT bitsSeek(Bitstream* Bits, const uint32_t Address)
{

    if (!Bits)
		return 0;

	Bits->Byte = Address;
	if (Bits->Byte > Bits->dataLen)
		Bits->Byte = Bits->dataLen;
    Bits->Bit = 0;

	return Bits->Byte;

}


// Skip a number of bytes
//
void EXPORT bitsSkipBytes(Bitstream* Bits, const uint32_t Bytes)
{

    if (!Bits)
		return;

	if (Bits->Byte + Bytes > Bits->dataLen)
        Bits->Byte = Bits->dataLen;
    else
        Bits->Byte += Bytes;

}


// Open a bitstream from a file
//
Bitstream* EXPORT bitsOpenFile(const char* fileName)
{

    Bitstream* Bits;
    FILE* F;


    F = fopen(fileName, "rb");
    if (!F)
        return NULL;

	Bits = bitsCreate();
	if (!Bits)
	{
		fclose(F);
		return NULL;
	}

	// Get the length of the file
    fseek(F, 0, SEEK_END);
	Bits->dataLen = ftell(F);
    fseek(F, 0, SEEK_SET);

	// Read the file into our buffer
	Bits->ptrData = malloc(Bits->dataLen);
	if (fread(Bits->ptrData, 1, Bits->dataLen, F) == 0)
	{
		fclose(F);
		bitsDestroy(Bits);
		return NULL;
	}

	fclose(F);
    return Bits;

}